import os, requests

from typing import List, Tuple, AnyStr


def _get_vars(params: List[Tuple], kw: dict) -> List[AnyStr]:
    result = []

    for kw_name, env_name in params:
        env_name = f'MY_{env_name}'
        if kw_name in kw:
            result.append(kw[kw_name])
        elif env_name in os.environ:
            result.append(os.environ[env_name])
        else:
            raise RuntimeError("missing parameter '%s' or variable '%s'" % (kw_name, env_name))

    return result


def telegram_notify(text: str, parse_mode: str = None, **kwargs):
    chat_id, token = _get_vars([
        ('chat_id', 'TELEGRAM_CHAT_ID'),
        ('token', 'TELEGRAM_TOKEN')
    ], kwargs)

    data = {
        'chat_id': chat_id,
        'text': text
    }
    if parse_mode is not None:
        data['parse_mode'] = parse_mode

    r = requests.post('https://api.telegram.org/bot%s/sendMessage' % token, data=data)

    if r.status_code != 200:
        raise RuntimeError("telegram returned %d" % r.status_code)



import os, json, atexit

from .functions import _get_vars


class State:
    def __init__(self, default: dict = None, **kwargs):
        file, = _get_vars([
            ('file', 'STATE_FILE')
        ], kwargs)

        if default is None:
            default = {}
        elif type(default) is not dict:
            raise TypeError('default must be dictionary')

        if not os.path.exists(file):
            self._data = default
        else:
            with open(file, 'r') as f:
                self._data = json.loads(f.read())

        self._file = file
        atexit.register(self.__cleanup)

    def __cleanup(self):
        if hasattr(self, '_file'):
            with open(self._file, 'w') as f:
                f.write(json.dumps(self._data))
        atexit.unregister(self.__cleanup)

    def __del__(self):
        if 'open' in __builtins__:
            self.__cleanup()

    def __getitem__(self, key):
        return self._data[key]

    def __setitem__(self, key, value):
        self._data[key] = value

    def __delitem__(self, key):
        if key in self._data:
            del self._data[key]